﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WinExt.Device;
using WinExt.Registry;
using WinExt.WinAPI;
using WinExt.WinIO;

namespace WinExt.ServiceApplication
{
    /// <summary>
    /// EMService background application.
    /// </summary>
    public partial class FrmMain : Form
    {
        private RuggedPad62H XDevice;
        private Register register;
        private readonly string FKEY_REGEDITKEY_NAME = @"StartProcessPath";
        private readonly string FKEY_REGEDITKEY_SUBKEY = @"SOFTWARE\XDevice\FKey";
        private readonly RegDomain FKEY_REGEDITKEY_DOMAIN = RegDomain.LocalMachine;

        public FrmMain()
        {
            InitializeComponent();
            //Hide the from
            this.ShowInTaskbar = false;
            this.WindowState = FormWindowState.Minimized;
            this.FormBorderStyle = FormBorderStyle.None;
            //Initial members
            XDevice = new RuggedPad62H();
            XDevice.KeyListener.OnKeyEvent += KeyListener_OnKeyEvent;
            XDevice.KeyListener.Start(RuggedPad62H.FKEY);
            register = new Register();
            register.Domain = FKEY_REGEDITKEY_DOMAIN;
            register.SubKey = FKEY_REGEDITKEY_SUBKEY;
            register.RegeditKey = FKEY_REGEDITKEY_NAME;
        }

        /// <summary>
        /// Listen the key press event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="onPressedKeys"></param>
        private void KeyListener_OnKeyEvent(object sender, Key[] onPressedKeys)
        {
            if (onPressedKeys == null && onPressedKeys.Length <= 0) return;
            //On click(one key)
            if (onPressedKeys.Length == 1 && onPressedKeys[0] != null)
            {
                //appendtext(string.Format("0x{0:X}", onPressedKeys[0].KeyCode));
                switch (onPressedKeys[0].KeyCode)
                {
                    case RuggedPad62H.FKEY:
                        FKey_OnKeyEvent();
                        break;
                    default:
                        break;
                }
            }
        }
        /// <summary>
        /// hot key(Alt+Tab:hide the form)
        /// </summary>
        /// <param name="value"></param>
        protected override void SetVisibleCore(bool value)
        {
            base.SetVisibleCore(false);
        }
        /// <summary>
        /// Recycling resources when the form is closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            XDevice.Dispose();
            WinIo.Dispose();
        }
        /// <summary>
        /// Key event callback method 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="keyValue"></param>
        private void FKey_OnKeyEvent()
        {
            //Button is pressed to get the registry Fkey wake up the name of the process, to detect whether the process has been running 
            if (register.IsRegeditKeyExist(FKEY_REGEDITKEY_NAME, FKEY_REGEDITKEY_SUBKEY, FKEY_REGEDITKEY_DOMAIN))
            {
                object fKeyStartProcessPathObj = register.ReadRegeditKey(FKEY_REGEDITKEY_NAME, FKEY_REGEDITKEY_SUBKEY, FKEY_REGEDITKEY_DOMAIN);
                string fKeyStartProcessPath = fKeyStartProcessPathObj != null ? fKeyStartProcessPathObj.ToString() : null;
                string fKeyStartProcessName = fKeyStartProcessPath != null ? System.IO.Path.GetFileNameWithoutExtension(fKeyStartProcessPath) : null;
                //If the Fkey in the registry is not empty, the process path exists 
                if (fKeyStartProcessPath != null && File.Exists(fKeyStartProcessPath))
                {
                    //Get the path of start app.
                    System.Diagnostics.Process fKeyStartProcess = null;
                    System.Diagnostics.Process[] processes = System.Diagnostics.Process.GetProcessesByName(fKeyStartProcessName);
                    for (int i = 0; i < processes.Length; i++)
                    {
                        if (processes[i] != null)
                        {
                            fKeyStartProcess = processes[i];
                            break;
                        }
                    }
                    //If the process does not exist, start the process 
                    if (fKeyStartProcess == null)
                    {
                        //Current process instance 
                        FileInfo fileInfo = new FileInfo(fKeyStartProcessPath);
                        //Directory of the current process 
                        DirectoryInfo directoryInfo = fileInfo != null ? fileInfo.Directory:null;
                        //The working directory of the current process 
                        string processDirectoryName = directoryInfo != null ? directoryInfo.FullName : null;
                        //Instance of starting information 
                        ProcessStartInfo processStartInfo = new ProcessStartInfo(fKeyStartProcessPath);
                        //Set the current working path 
                        processStartInfo.WorkingDirectory = processDirectoryName;
                        //Start process 
                        Process.Start(processStartInfo); 
                    }
                    else
                    {
                        User32.SwitchToThisWindow(fKeyStartProcess.MainWindowHandle, true);
                        User32.SetWindowPos(fKeyStartProcess.MainWindowHandle, User32.HWND_TOPMOST, 0, 0, 0, 0, User32.SWP_NOMOVE | User32.SWP_NOSIZE );
                    }
                }
            }
            else
            {
                register.WriteRegeditKey(FKEY_REGEDITKEY_NAME, "");
            }
        }
    }
}
