﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;
using TaskScheduler;
using WinExt.Registry;

namespace WinExt.ServiceApplicationUI
{
    public partial class FrmMain : Form
    {
        private Register register;
        private readonly string FKEY_REGEDITKEY_NAME = @"StartProcessPath";
        private readonly string FKEY_REGEDITKEY_SUBKEY = @"SOFTWARE\XDevice\FKey";
        private readonly RegDomain FKEY_REGEDITKEY_DOMAIN = RegDomain.LocalMachine;
        private readonly string SERVICE_STATUS_RUNNING = "Started";
        private readonly string SERVICE_STATUS_STOPED = "Stoped";
        private readonly string SERVICE_STATUS_UNINSTALLED = "UnInstalled;";
        private IRegisteredTask GetKeyServiceTask
        {
            get
            {
                TaskSchedulerClass taskScheduler = new TaskSchedulerClass();
                taskScheduler.Connect(null, null, null, null);
                ITaskFolder folder = taskScheduler.GetFolder("\\");
                IRegisteredTaskCollection registeredTaskCollection = folder.GetTasks(1);
                IRegisteredTask registeredTask = null;
                foreach (IRegisteredTask item in registeredTaskCollection)
                {
                    if (item!=null&&item.Name.Equals("StartKeyServiceTask"))
                    {
                        registeredTask = item;
                        break;
                    }
                }
                return registeredTask;
            }
        }
        private Process GetEMServiceApplication {
            get {
                Process process = null;
                Process[] processes = Process.GetProcessesByName("WinExt.KeyService");
                if (processes!=null&& processes.Length>0)
                {
                    foreach (Process item in processes)
                    {
                        process = item;
                        break;
                    }
                }
                return process;
            }
        }
        public FrmMain()
        {
            InitializeComponent();
            System.Windows.Forms.Control.CheckForIllegalCrossThreadCalls = false;
            register = new Register();
            register.Domain = FKEY_REGEDITKEY_DOMAIN;
            register.SubKey = FKEY_REGEDITKEY_SUBKEY;
            register.RegeditKey = FKEY_REGEDITKEY_NAME;
            //Read the path of fKeyStartProcess from register 
            if (register.IsRegeditKeyExist(FKEY_REGEDITKEY_NAME, FKEY_REGEDITKEY_SUBKEY, FKEY_REGEDITKEY_DOMAIN))
            {
                object fKeyStartProcessPathObj = register.ReadRegeditKey(FKEY_REGEDITKEY_NAME, FKEY_REGEDITKEY_SUBKEY, FKEY_REGEDITKEY_DOMAIN).ToString();
                string fKeyStartProcessPath = fKeyStartProcessPathObj != null ? fKeyStartProcessPathObj.ToString() : null;
                txt_fKeyStartProcess.Text = fKeyStartProcessPath;
            }
        }
        private void btn_service_status_Click(object sender, EventArgs e)
        {
            IRegisteredTask keyServiceTask = GetKeyServiceTask;
            if (keyServiceTask != null)
            {
                btn_install.Enabled = false;
                btn_uninstall.Enabled = true;
                pn_statusManager.Enabled = true;

                Process emServiceApplication = GetEMServiceApplication;
                btn_start.Enabled = emServiceApplication != null ? false : true;
                btn_stop.Enabled = emServiceApplication != null ? true : false;
                btn_service_status.Text = emServiceApplication != null ? SERVICE_STATUS_RUNNING : SERVICE_STATUS_STOPED;
            }
            else
            {
                btn_install.Enabled = true;
                btn_uninstall.Enabled = false;
                pn_statusManager.Enabled = false;
                btn_service_status.Text = SERVICE_STATUS_UNINSTALLED;
            }
        }

        private void btn_install_Click(object sender, EventArgs e)
        {
            string currentDirectory = System.Environment.CurrentDirectory;
            //Update the startup path in the task schedule 
            update(currentDirectory + @"\Service\StartKeyServiceTask.xml", currentDirectory + @"\Service\WinExt.KeyService.exe");
            //Deployment task schedule 
            System.Environment.CurrentDirectory = currentDirectory + "\\Service";
            Process process = new Process();
            process.StartInfo.UseShellExecute = false;
            process.StartInfo.FileName = "Install.bat";
            process.StartInfo.CreateNoWindow = true;
            process.Start();
            process.Refresh();
            System.Environment.CurrentDirectory = currentDirectory;
        }

        private void btn_uninstall_Click(object sender, EventArgs e)
        {
            btn_stop_Click(sender, e);
            string CurrentDirectory = System.Environment.CurrentDirectory;
            System.Environment.CurrentDirectory = CurrentDirectory + "\\Service";
            Process process = new Process();
            process.StartInfo.UseShellExecute = false;
            process.StartInfo.FileName = "Uninstall.bat";
            process.StartInfo.CreateNoWindow = true;
            process.Start();
            process.Refresh();
            System.Environment.CurrentDirectory = CurrentDirectory;
        }

        private void btn_start_Click(object sender, EventArgs e)
        {
            IRegisteredTask registeredTask = GetKeyServiceTask;
            if (registeredTask != null&& registeredTask.State!= _TASK_STATE.TASK_STATE_RUNNING)
            {
                registeredTask.Run(null);
            }
        }

        private void btn_stop_Click(object sender, EventArgs e)
        {
            IRegisteredTask registeredTask = GetKeyServiceTask;
            if (registeredTask != null && registeredTask.State == _TASK_STATE.TASK_STATE_RUNNING)
            {
                registeredTask.Stop(0);
            }
        }

        private void btn_fKeyStartProcess_browser_Click(object sender, EventArgs e)
        {
            openFileDialog.InitialDirectory = "\\";
            openFileDialog.Filter = "exe|*.exe|lnk|*.lnk";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;
            if (openFileDialog.ShowDialog() == DialogResult.OK) { 
                txt_fKeyStartProcess.Text = openFileDialog.FileName;
            }
        }

        private void btn_fKeyStartProcess_set_Click(object sender, EventArgs e)
        {
            if (register.WriteRegeditKey(FKEY_REGEDITKEY_NAME, txt_fKeyStartProcess.Text))
            {
                MessageBox.Show("Set Succeed!", "Notification", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Set Failed!", "Notification", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
      
        private void timer_checkServiceStatus_Tick(object sender, EventArgs e)
        {
            btn_service_status_Click(sender, e);
        }

        private void update(string path, string execPath) {

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(path);
            if (xmlDoc!=null)
            {
                XmlElement xmlRoot = xmlDoc.DocumentElement;
                if (xmlRoot!=null)
                {
                    XmlNamespaceManager xmlns = new XmlNamespaceManager(xmlDoc.NameTable);
                    xmlns.AddNamespace("task", "http://schemas.microsoft.com/windows/2004/02/mit/task");
                    XmlNode commandNode=  xmlRoot.SelectSingleNode("/task:Task/task:Actions/task:Exec/task:Command", xmlns);
                    if (commandNode!=null)
                    {
                        commandNode.InnerText = execPath;
                        xmlDoc.Save(path);
                    }
                }
            }
        }
    }
}
